#~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*#
# featurestats.tcl                                                         #
#                                                                          #
# Description:                                                             #
#                                                                          #
#     This Tcl/Tk script defines an interface for visualizing the feature  #
#     vector that is associated with each hand-written symbol, as well as  #
#     the statistics associated with each class of trained symbols; that   #
#     is, it graphically depicts the mean and standard deciation of each   #
#     feature (of a given class), and superimposes the specific feature    #
#     vector of a newly-written symbol.                                    #
#                                                                          #
# Changes:                                                                 #
#                                                                          #
#     09/20/99  arvo    Split off from Train.tcl.                          #
#                                                                          #
#--------------------------------------------------------------------------#
# Copyright (C) 1999, James Arvo                                           #
#                                                                          #
# This program is free software; you can redistribute it and/or modify it  #
# under the terms of the GNU General Public License as published by the    #
# Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      #
#                                                                          #
# This program is distributed in the hope that it will be useful, but      #
# WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    #
# any particular purpose.  See the GNU General Public License for more     #
# details.                                                                 #
#                                                                          #
#~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*#

proc ShowFeatureStats { num_features } {
  global stats
  if [catch "toplevel .stats -visual best"] then return
  wm title .stats "Feature Statistics"
  set stats(frac) 0
  set w  [expr $stats(column-width) * $num_features]
  set h  $stats(chart-height)

  frame  .stats.top       -relief flat  -borderwidth 0
  frame  .stats.bot       -relief flat  -borderwidth 0
  button .stats.bot.left  -text "<<"   -command {SendMessage change_stats -1}
  button .stats.bot.reset -text "best" -command {SendMessage change_stats  0}
  button .stats.bot.right -text ">>"   -command {SendMessage change_stats  1}
  label  .stats.bot.comm  -background white -relief ridge -bd 3 -width 65
  label  .stats.bot.symb  -background white -relief ridge -bd 3 -width 30
  label  .stats.bot.prob  -background white -relief ridge -bd 3 -width 12
  button .stats.bot.close -relief raised -bd 3 -command HideFeatureStats -text Close

  canvas .stats.top.canv     \
    -relief ridge            \
    -borderwidth 3           \
    -width $w                \
    -height $h               \
    -background white        \
    -closeenough 3           \
    -confine false           \
    -scrollregion "0 0 $w [expr 3 * $h]"

  canvas .stats.top.axis     \
    -relief ridge            \
    -borderwidth 3           \
    -width 20                \
    -height $h               \
    -background white        \
    -confine false           \
    -scrollregion "0 0 $w [expr 3 * $h]"

  canvas .stats.top.nums     \
    -relief ridge            \
    -borderwidth 3           \
    -width $w                \
    -height 20               \
    -background white  

  scale  .stats.top.scr      \
    -from -1.0 -to 1.0       \
    -resolution 0.01         \
    -orient vertical         \
    -width 12                \
    -showvalue false         \
    -command StatsSlider

  pack .stats.top -side top  -fill x -expand true
  grid .stats.top.axis .stats.top.canv .stats.top.scr -sticky ns
  grid .stats.top.nums -column 1
  pack .stats.bot       -side top  -fill x -expand true
  pack .stats.bot.left  -side left         -padx 4 -pady 4
  pack .stats.bot.reset -side left         -padx 4 -pady 4
  pack .stats.bot.right -side left         -padx 4 -pady 4
  pack .stats.bot.comm  -side left -fill y -padx 4 -pady 4 
  pack .stats.bot.symb  -side left -fill y -padx 4 -pady 4
  pack .stats.bot.prob  -side left -fill y -padx 4 -pady 4 
  pack .stats.bot.close -side left -fill both -expand true -padx 8 -pady 8

  set step $stats(column-width)
  set half [expr $step / 2]
  set y_top [expr -3 * $h]
  set y_bot [expr  4 * $h]
  for {set i 0} {$i < $num_features} {incr i} {
    set x [expr ($i + 1) * $stats(column-width)]
   .stats.top.canv create line $x $y_top $x $y_bot -fill #e0e0e0
    }
  for {set i 0} {$i < $num_features} {incr i} {
    set x [expr $i * $stats(column-width) + $half]
   .stats.top.nums create text $x 15 -text [expr $i + 1]
    }
  for {set i -12} {$i <= 12} {incr i} {
    set y [expr $stats(offset) + $i * $stats(scale)]
    set line_color #d8d8d8
    set text_color black
    if { $i == 0 } then { set line_color red; set text_color red }
   .stats.top.axis create text 12 $y -text [expr -$i] -fill $text_color
   .stats.top.canv create line 0 $y $w $y -fill $line_color
    }
  }

proc StatsSlider { frac } {
  global stats
  set stats(frac) $frac
 .stats.top.canv yview moveto $frac
 .stats.top.axis yview moveto $frac
  }

proc HideFeatureStats {} {
  destroy .stats
  SendMessage show_stats off
  }

proc PostFeatureStats { header features data } {
  global stats
  set col 0
 .stats.top.canv delete STATS
  foreach d $data {
    set mean [lindex $d 0]
    set stdd [lindex $d 1]
    set min  [lindex $d 2]
    set max  [lindex $d 3]
    ShowFeature $col $mean $stdd $min $max
    incr col
    }
  set col 0
  foreach f $features {
    DrawDot $col [lindex $f 0] [lindex $f 1]
    incr col
    }
  set symbol [lindex $header 0]
  set file   [lindex $header 1]
  set prob   [lindex $header 2]
 .stats.bot.symb config -text "$symbol ($file)"
 .stats.bot.prob config -text $prob
 .stats.top.canv yview moveto $stats(frac)
  }

proc ShowFeature { col mean stdd min max } {
  global stats
  set step   $stats(column-width)
  set scale  $stats(scale)
  set yc     $stats(offset)
  set height [expr $stats(chart-height) - $yc]
  set x0     [expr $col * $step + 5]
  set x      [expr ($col + 0.5) * $step]
  set x1     [expr ($col + 1) * $step - 5]
  set y0     [expr $height - ($min  * $scale)]
  set y      [expr $height - ($mean * $scale)]
  set y1     [expr $height - ($max  * $scale)]
 .stats.top.canv create rectangle $x0 $y0 $x1 $y1 -fill gray90 -outline gray80 -tag STATS
  DrawDotWithBars $x $y [expr $stdd * $scale] blue
  }

proc DrawDotWithBars { x y stdd color } {
  set w 3
  set y0 [expr $y - $stdd]
  set y1 [expr $y + $stdd]
  set x0 [expr $x - $w]
  set x1 [expr $x + $w]
 .stats.top.canv create line $x  $y0 $x  $y1 -fill $color -width 2 -tag STATS
 .stats.top.canv create line $x0 $y0 $x1 $y0 -fill $color -width 2 -tag STATS
 .stats.top.canv create line $x0 $y1 $x1 $y1 -fill $color -width 2 -tag STATS
 .stats.top.canv create oval $x0 [expr $y - $w] $x1 [expr $y + $w] \
    -fill $color -outline $color -tag STATS
  }

proc DrawDot { col value comment } {
  global stats
  set w     5
  set h     3
  set step  $stats(column-width)
  set scale $stats(scale)
  set yc    $stats(offset)
  set x     [expr ($col + 0.5) * $step]
  set y     [expr $stats(chart-height) - ($value * $scale + $yc)]
  set dot   "dot$col"
 .stats.top.canv create rectangle \
    [expr $x - $w] [expr $y - $h] [expr $x + $w] [expr $y + $h] \
    -outline red -width 2 -tag $dot
 .stats.top.canv addtag STATS withtag $dot
 .stats.top.canv bind $dot <Enter> "EnterDot $dot {$comment}"
 .stats.top.canv bind $dot <Leave> "LeaveDot $dot"
  }

proc EnterDot { dot comment } {
 .stats.top.canv itemconfig $dot -outline green -width 6
 .stats.bot.comm config -text "$comment"
  }

proc LeaveDot { dot } {
 .stats.top.canv itemconfig $dot -outline red -width 2
 .stats.bot.comm config -text ""
  }

