#~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*#
# profile.tcl                                                              #
#                                                                          #
# Description:                                                             #
#                                                                          #
#     This Tcl/Tk script defines pop-up bar chart that depicts the         #
#     relative confidence levels of the top-ten candidates for any         #
#     recognized character.                                                #
#                                                                          #
# Changes:                                                                 #
#                                                                          #
#     09/20/99  arvo    Split off from Train.tcl.                          #
#                                                                          #
#--------------------------------------------------------------------------#
# Copyright (C) 1999, James Arvo                                           #
#                                                                          #
# This program is free software; you can redistribute it and/or modify it  #
# under the terms of the GNU General Public License as published by the    #
# Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      #
#                                                                          #
# This program is distributed in the hope that it will be useful, but      #
# WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    #
# any particular purpose.  See the GNU General Public License for more     #
# details.                                                                 #
#                                                                          #
#~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*#

proc RaiseProfileWindow {} {
  global colors canv font
  if [catch "toplevel .profile -visual best"] then return
  wm title .profile "Recognition Profile"
  canvas .profile.canv -height $canv(prof-height) -width $canv(prof-width) \
    -background white -relief groove -borderwidth 3 -cursor hand2
  frame  .profile.bar -relief flat -borderwidth 0
  label  .profile.bar.name  -relief sunken -borderwidth 4 -font $font(normal) -background white
  label  .profile.bar.prob  -relief sunken -borderwidth 4 -font $font(normal) -background white -width 11
  button .profile.bar.close \
    -text Close             \
    -relief raised          \
    -borderwidth 3          \
    -font $font(normal)     \
    -command LowerProfileWindow 
  pack .profile.canv      -side top  -padx 8 -pady 8
  pack .profile.bar       -side top  -padx 8 -pady 0 -fill x -expand true
  pack .profile.bar.name  -side left -padx 3 -pady 2 -fill x -expand true
  pack .profile.bar.prob  -side left -padx 3 -pady 2
  pack .profile.bar.close -side left -padx 3 -pady 2
  SendMessage show_profile
  }

proc LowerProfileWindow {} {
  catch [destroy .profile]
  SendMessage hide_profile
  }

# Draw a bar chart that shows the relative confidence of each
# choice determined by the recognizer.  This is a visual version
# of the "profile" handed back by the recognizer.  The list expected
# by this procedure consists of {lett weight prob} triples, where
# "lett" is the name of the letter, "weight" is the normalized
# probablility (always 1 for the most likely choice), and "prob"
# is the actual probability (or confidence).
#
proc UpdateProfile { list } {
  global canv font colors profile
 .profile.canv delete ALL
  set L  [llength $list]
  set s  4
  set w  [expr $canv(prof-width) / $L - $s] 
  set x0 $s
  set y1 $canv(prof-height)
  set i  0
  foreach pair $list {
    incr i
    set  lett   [lindex $pair 0]
    set  weight [lindex $pair 1]
    set  prob   [lindex $pair 2]
    set  lab    $lett
    set x1 [expr $x0 + $w]
    set y0 [expr $y1 * ( 1.0 - $weight )]
    if { [string length $lett] > 1 } then { set lab [string range $lett 0 2] }
   .profile.canv create rectangle $x0 [expr $y0 + $s] $x1 [expr $y1 - $s] \
      -fill $colors(bar-normal) -tags "ALL B$i"
   .profile.canv create text [expr $x0 + $w/2] [expr $y0 + 10 + $s] \
     -font $font(normal) -tag ALL -fill white -text "$lab"
   .profile.canv bind "B$i" <Enter> "EnterBar B$i \"$lett\" $prob"
   .profile.canv bind "B$i" <Leave> "LeaveBar B$i"
    set x0 [expr $x1 + $s]
    if { $i == 1 } then {
      set profile(name) $lett
      set profile(prob) $prob
      EnterBar none $lett $prob
      }
    }
  }

proc EnterBar { id name prob } {
  global colors
  if { $id == "none" } then { 
    set labcolor white
    set barcolor $colors(bar-normal) 
  } else { 
    set labcolor $colors(bar-hilite) 
    set barcolor $colors(bar-hilite) 
    }
 .profile.canv itemconfigure $id -fill $barcolor
 .profile.bar.name configure -text $name -background $labcolor
 .profile.bar.prob configure -text $prob -background $labcolor
  }

proc LeaveBar { id } {
  global colors profile
 .profile.canv itemconfigure $id -fill $colors(bar-normal)
 .profile.bar.name configure -text $profile(name) -background white
 .profile.bar.prob configure -text $profile(prob) -background white
  }

