/***************************************************************************
* Grouper.h                                                                *
*                                                                          *
* Basic class definitions for the stroke grouping algorithm.               *
*                                                                          *
*   Changes:                                                               *
*                                                                          *
*       10/20/99  arvo    Initial coding.                                  *
*                                                                          *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo                                           *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#ifndef __GROUPER_INCLUDED__
#define __GROUPER_INCLUDED__

#include <util/Multiline.h>
#include <rec/Symbol.h>

static const int MaxSymbolStrokes = 4;

// The "Grouper" class is a purely virtural class that serves only
// as the base for symbol Groupers.  Each Grouper must define a new 
// class by deriving from this one and overriding the "<<" operator
// and the GetCharacter method.  Here's what the methods do:
//
//   operator<<    Adds a new stroke to the queue of pending strokes.
//   GetCharacter  Gets next collection of strokes unless value is NULL.
//
// When the "force" parameter to GetCharacter is non-zero, it means
// that no more strokes will be supplied, so characters must be
// grouped until the queue is empty.
//
class Grouper {

    public:
      Grouper() {}
      virtual ~Grouper() {}

    public: // These virtual methods are optional.
      virtual void SetRecognizer( Recognizer & ) {}

    public: // These purely virtual methods MUST be supplied.
      virtual void    Clear() = 0;
      virtual void    operator<<( const Multiline & ) = 0;
      virtual Symbol *GetCharacter( Multiline &, int force = 0 ) = 0;

    };

#endif





