/*****************************************************************************
** 
**   MODULE NAME  Delaunay.h
**
**   DESCRIPTION
**
**
**   HISTORY
**      Name    Date       Description
**
**      arvo    12/29/93   Copied from Dani Lischinski
**
**   (c) Copyright 1993
**       Program of Computer Graphics, Cornell University, Ithaca, NY
**       ALL RIGHTS RESERVED
**
*****************************************************************************/

#ifndef DELAUNAYB_INCLUDED
#define DELAUNAYB_INCLUDED

#include <util/Vec2.h>
#include <util/Multiline.h>
#include <util/quadedge.h>


class DelaunayMesh {
    public:
        DelaunayMesh( const Box2d & );
        DelaunayMesh();
       ~DelaunayMesh();
        void Add( const Multiline          &, float value = 0.0  );
        void Add( const Vec2               &, float value = 0.0  );
        void Add( const Vec2 &, const Vec2 &, float value1 = 0.0, float value2=0.0  );
        void Draw ( DrawLine_t func );
        void Shade( DrawTri_t  func );
        void TraverseTris ( TraverseTri_t   func, void * );
        void TraverseEdges( TraverseEdge_t  func, void * );
        void Clear();
        void Clear( const Box2d &box );
        void SetTol( float tol ) { SkipTol = tol; }
        void* GetMesh() { return mesh; }
    private:
        void *mesh;
        Box2d  box;
        float SkipTol;
    };

#endif



