/******************************************************************************
**
** $Header: /ufs/arvogroup/users/kevin/cvsroot/CIT4.3/include/util/quadedge.h,v 1.2 2001/02/12 04:53:19 kevin Exp $
**
** $Log: quadedge.h,v $
** Revision 1.2  2001/02/12 04:53:19  kevin
**
**
** Modified to work under linux
**
** Revision 1.1.1.1  2001/02/12 04:36:06  kevin
** Initial
**
** Revision 1.1  1999/12/24 04:40:01  novins
** Initial revision
**
** Revision 1.6  1999/10/10 10:59:49  novins
** *** empty log message ***
**
** Revision 1.5  1999/10/10 06:23:31  novins
** *** empty log message ***
**
** Revision 1.4  1999/10/10 02:35:04  novins
** *** empty log message ***
**
** Revision 1.3  1999/10/10 01:11:09  novins
** *** empty log message ***
**
** Revision 1.2  1999/10/09 04:21:03  novins
** *** empty log message ***
**
** Revision 1.1  1999/10/08 08:10:30  novins
** Initial revision
**
 * Revision 1.8  93/06/14  14:47:03  14:47:03  danix (Dani Lischinski)
 * 
 * Revision 1.1  92/10/16  09:20:02  09:20:02  danix (Dani Lischinski)
 * Initial revision
 * 
**
******************************************************************************/

#ifndef QUADEDGE_H
#define QUADEDGE_H

#include <math.h>
#include <stdlib.h>
#include <stream.h>
#include <assert.h>
#include <iostream.h>

struct line_t {
    float x1, y1;
    float x2, y2;
    };

struct triangle_t {
    float v1, x1, y1;  // Value, x-coord, y-coord.
    float v2, x2, y2;
    float v3, x3, y3;
    };

typedef void (*DrawLine_t)( const line_t     &line );
typedef void (*DrawTri_t )( const triangle_t &tri  );


/*****************************************************************************/
/** common.h                                                                **/
/*****************************************************************************/

#ifndef TRUE
#define FALSE           0
#define TRUE            1
#endif

typedef int             Boolean;

#ifndef ON
#define OFF             0
#define ON              1
#endif

typedef int             Switch;

#define NIL(type)       ((type*)0)

#ifndef MAX
#define MAX(a, b)       ((a) >= (b) ? (a) : (b))
#define MIN(a, b)       ((a) <= (b) ? (a) : (b))
#endif

#ifndef ABS
#define ABS(a)          ((a) >= 0 ? (a) : -(a))
#endif
#define SGN(a)                  ((a) > 0 ? 1 : ((a) < 0 ? -1 : 0))

#define forever                 for(;;)

inline int max(int a, int b)
{
        return (a >= b) ? a : b;
}

inline int min(int a, int b)
{
        return (a < b) ? a : b;
}

inline int max(int a, int b, int c)
{
        return (a >= b) ? (a >= c ? a : c)
                                        : (b >= c ? b : c);
}

inline int min(int a, int b, int c)
{
        return (a < b) ? (a < c ? a : c)
                                   : (b < c ? b : c);
}

inline float max(float a, float b)
{
        return (a >= b) ? a : b;
}

inline float min(float a, float b)
{
        return (a < b) ? a : b;
}

inline float max(float a, float b, float c)
{
        return (a >= b) ? (a >= c ? a : c)
                                        : (b >= c ? b : c);
}

inline float min(float a, float b, float c)
{
        return (a < b) ? (a < c ? a : c)
                                   : (b < c ? b : c);
}

inline double max(double a, double b)
{
        return (a >= b) ? a : b;
}

inline double min(double a, double b)
{
        return (a < b) ? a : b;
}

inline double max(double a, double b, double c)
{
        return (a >= b) ? (a >= c ? a : c)
                                        : (b >= c ? b : c);
}

inline double min(double a, double b, double c)
{
        return (a < b) ? (a < c ? a : c)
                                   : (b < c ? b : c);
}

inline int imin(double a, double b, double c)
{
        return (a < b) ? (a < c ? 0 : 2)
                                   : (b < c ? 1 : 2);
}

typedef float Sreal;
typedef double Lreal;

typedef char *charPtr;

#ifdef NDEBUG
#       define Assert(e)        (void(0))
#       define Warning(msg)     (void(0))
#       define Error(msg)       (void(0))
#else
#       define Assert(e)        assert(e)
#       define Warning(msg) \
                (cerr << "Warning: " << msg \
                          << ", file " << __FILE__ \
                          << ", line " << __LINE__ << "\n")
#       define Error(msg) \
                ((cerr << "Error: " << msg \
                           << ", file " << __FILE__ \
                           << ", line " << __LINE__ << "\n"), exit(1))
#endif
#define Abort(msg) \
                ((cerr << "Fatal Error: " << msg \
                           << ", file " << __FILE__ \
                           << ", line " << __LINE__ << "\n"), abort())
        

/*****************************************************************************/
/** geom2d.h                                                                **/
/*****************************************************************************/

#define EPS 1e-6
#define X 0
#define Y 1
#define Z 2

typedef Lreal Real;

class Vector2d {
public:
        Real x, y;
        Vector2d()                                      { x = 0; y = 0; }
        Vector2d(Real a, Real b)        { x = a; y = b; }
        Real& operator[](int i)                                 { return ((Real *)this)[i]; }
        const Real& operator[](int i) const     { return ((Real *)this)[i]; }
        Real norm() const;
        void normalize();
        Boolean operator==(const Vector2d&) const;
        Vector2d operator+(const Vector2d&) const;
        Vector2d operator-(const Vector2d&) const;
        Real     operator|(const Vector2d&) const;
        friend Vector2d operator*(Real, const Vector2d&);
        friend Vector2d operator/(const Vector2d&, Real);
        friend istream& operator>>(istream&, Vector2d&);
        friend ostream& operator<<(ostream&, const Vector2d&);
};

typedef Vector2d Point2d;

class Line {
public:
        Line()  {}
        Line(const Point2d&, const Point2d&);
        void set(const Point2d&, const Point2d&);
        Real eval(const Point2d&) const;
        int classify(const Point2d&) const;
        Point2d intersect(const Point2d&, const Point2d&) const;
private:
        Real a, b, c;
};

// Vector2d:

inline Real Vector2d::norm() const
{
        return sqrt(x * x + y * y);
}

inline void Vector2d::normalize()
{
        Real len;

        if ((len = sqrt(x * x + y * y)) == 0.0)
                cerr << "Vector2d::normalize: Division by 0\n";
        else {
                x /= len;
                y /= len;
        }
}

inline Vector2d Vector2d::operator+(const Vector2d& v) const
{
        return Vector2d(x + v.x, y + v.y);
}

inline Vector2d Vector2d::operator-(const Vector2d& v) const
{
        return Vector2d(x - v.x, y - v.y);
}

inline Boolean Vector2d::operator==(const Vector2d& v) const
{
        return (*this - v).norm() <= EPS;
}

inline Real Vector2d::operator|(const Vector2d& v) const
{
        return x * v.x + y * v.y;
}

inline Vector2d operator*(Real c, const Vector2d& v)
{
        return Vector2d(c * v.x, c * v.y);
}

inline Vector2d operator/(const Vector2d& v, Real c)
{
        return Vector2d(v.x / c, v.y / c);
}

inline ostream& operator<<(ostream& os, const Vector2d& v)
{
        os << '(' << v.x << ", " << v.y << ')';
        return os;
}

inline istream& operator>>(istream& is, Vector2d& v)
{
        is >> v.x >> v.y;
        return is;
}

// Line:

inline Line::Line(const Point2d& p, const Point2d& q)
// Computes the normalized line equation through the
// points p and q.
{
        Vector2d t = q - p;
        Real len = t.norm();
        a =   t.y / len;
        b = - t.x / len;
        c = -(a*p.x + b*p.y);
}

inline void Line::set(const Point2d& p, const Point2d& q)
{
        *this = Line(p, q);
}

inline Real Line::eval(const Point2d& p) const
// Plugs point p into the line equation.
{
        return (a * p.x + b* p.y + c);
}

inline Point2d Line::intersect(const Point2d& p1, const Point2d& p2) const
// Returns the intersection of the line with the segment (p1,p2)
{
        // assumes that segment (p1,p2) crosses the line
        Vector2d d = p2 - p1;
        Real t = - eval(p1) / (a*d[X] + b*d[Y]);
        return (p1 + t*d);
}

inline int Line::classify(const Point2d& p) const
// Returns -1, 0, or 1, if p is to the left of, on,
// or right of the line, respectively.
{
        Real d = eval(p);
        return (d < -EPS) ? -1 : (d > EPS ? 1 : 0);
}

inline Boolean operator==(const Point2d& point, const Line& line)
// Returns TRUE if point is on the line (actually, on the EPS-slab
// around the line).
{
        Real tmp = line.eval(point);
        return(ABS(tmp) <= EPS);
}

inline Boolean operator<(const Point2d& point, const Line& line)
// Returns TRUE if point is to the left of the line (left to
// the EPS-slab around the line).
{
        return (line.eval(point) < -EPS);
}


/*****************************************************************************/
/** llist.h - include file for linked lists/stacks/queues package           **/
/*****************************************************************************/

typedef void* Ldata;

class Llist;

class LlistNode {
        friend Llist;
private:
        Ldata data;
        LlistNode* next;
        LlistNode()                                             { data = 0; next = this; }
        LlistNode(Ldata d, LlistNode* n)        { data = d; next = n; }
};

typedef LlistNode *LlistPos;

class Llist {
private:
        int len;
        LlistNode *node;
public:
        Llist()                                                 { node = new LlistNode; len = 0; }
        ~Llist();
        Boolean empty() const           { return (node == node->next); }
        void insert(LlistPos, Ldata);
        void remove(LlistPos);
        LlistPos first() const                  { return node; }
        Boolean isEnd(LlistPos p) const         { return (p->next == node); }
        LlistPos next(LlistPos p) const         { return p->next; }
        Ldata retrieve(LlistPos p) const        { return p->next->data; }
        void store(LlistPos p, Ldata d)         { p->next->data = d; }
        int length() const                      { return len; }

};

inline void Llist::insert(LlistPos p, Ldata d)
{
        p->next = new LlistNode(d, p->next);
        len++;
}

inline void Llist::remove(LlistPos p)
{
        LlistNode *t = p->next;
        p->next = p->next->next;
        delete t;
        len--;
}

inline Llist::~Llist()
{
        while (!empty()) remove(first());
        delete node;
}


/*****************************************************************************/
/** quadedge.h                                                              **/
/*****************************************************************************/


class QuadEdge;

// typedef Point2d Data;

typedef enum {f_none,f_terminal,f_sleeve,f_junction,f_marked} face_type;
class Data : public Point2d {
    public:
       ~Data() {}
        Data() : Point2d() { value = 0.0; bdry = 0; }
        Data( const Point2d& p, float val = 0.0 ) : Point2d(p) { value = val; bdry = 0; }
        void  Value( float x ) { value = x; }
        float Value() const { return value; }
        void  OnBoundary() { bdry = 1; }
        int   BoundaryP () const { return bdry; }
    private:
        float value;
        int   bdry;   // Is this vertex on the outer boundary?
    };        



class Edge {
        friend QuadEdge;
        friend void Splice(Edge*, Edge*);
  private:
        int num;
        Edge *next;
        Data *data;
        face_type  ftype; // For Chordal Axis Transform
  public:
        Edge()  { data = 0; }
        Edge* Rot();
        Edge* invRot();
        Edge* Sym();
        Edge* Onext();
        Edge* Oprev();
        Edge* Dnext();
        Edge* Dprev();
        Edge* Lnext();  
        Edge* Lprev();
        Edge* Rnext();  
        Edge* Rprev();
        Data* Org();
        Data* Dest();
        const Point2d& Org2d() const;
        const Point2d& Dest2d() const;
        void  EndPoints(Data*, Data*);
        Boolean isConstrained();
        void Constrain();
        face_type  Ftype() const { return ftype; }
        void  Ftype( face_type x) { ftype = x; }
        QuadEdge* QEdge() const                                 { return (QuadEdge *)(this - num); }
};

typedef void (*TraverseEdge_t)( QuadEdge *, void * );
typedef void (*TraverseTri_t) ( Edge *,     void * );

typedef enum {e_none,e_internal,e_external,e_marked} edge_type;

class QuadEdge {
  private:
        Edge e[4];
        Boolean c;
        edge_type etype;
  public:
        QuadEdge(Boolean);
        Edge *edges()                   { return e; }
        Boolean isConstrained()         { return c; }
        void Constrain()                { c = TRUE; }
        void Draw( DrawLine_t );
        void DrawTriangle( DrawTri_t );
        edge_type  Etype() const { return etype; }
        void  Etype( edge_type x) { etype = x; }
        ~QuadEdge();
};

class Mesh {
  private:
        Edge *startingEdge;
        Llist edges;
        void DeleteEdge(Edge*);
        Edge *Connect(Edge*, Edge*);
        Edge *Locate(const Point2d&);
        Edge *BrutalLocate(const Point2d&);
        void SplitEdge(Edge*, const Point2d&);
        void Triangulate(Edge*);
  public:
        Mesh(const Point2d&, const Point2d&, const Point2d&);
        Mesh(const Point2d&, const Point2d&, const Point2d&, const Point2d&);
        Edge *MakeEdge(Boolean);
        Edge *MakeEdge(Data*, Data*, Boolean);
        Edge *InsertSite(const Point2d&, Real dist = EPS );
        Edge *InsertValue(const Point2d&, float value );
        Edge *InsertBoundaryPoint( const Point2d &x );
        void InsertEdge(const Point2d&, const Point2d&, float = 0.0, float = 0.0);
        int  numEdges() const   { return edges.length(); }
        void DrawLines    ( DrawLine_t ) const;
        void DrawTriangles( DrawTri_t  ) const;
        ~Mesh();

  public:
        void TraverseEdges    ( TraverseEdge_t, void * );
        void TraverseTris     ( TraverseTri_t , void * );
};

inline QuadEdge::QuadEdge(Boolean constrained = FALSE)
{
        e[0].num = 0, e[1].num = 1, e[2].num = 2, e[3].num = 3;
        e[0].next = &(e[0]); e[1].next = &(e[3]);
        e[2].next = &(e[2]); e[3].next = &(e[1]);
        c = constrained;
}

/************************* Edge Algebra *************************************/

inline Edge* Edge::Rot()
// Return the dual of the current edge, directed from its right to its left. 
{
        return (num < 3) ? this + 1 : this - 3;
}

inline Edge* Edge::invRot()
// Return the dual of the current edge, directed from its left to its right. 
{
        return (num > 0) ? this - 1 : this + 3;
}

inline Edge* Edge::Sym()
// Return the edge from the destination to the origin of the current edge.
{
        return (num < 2) ? this + 2 : this - 2;
}
        
inline Edge* Edge::Onext()
// Return the next ccw edge around (from) the origin of the current edge.
{
        return next;
}

inline Edge* Edge::Oprev()
// Return the next cw edge around (from) the origin of the current edge.
{
        return Rot()->Onext()->Rot();
}

inline Edge* Edge::Dnext()
// Return the next ccw edge around (into) the destination of the current edge.
{
        return Sym()->Onext()->Sym();
}

inline Edge* Edge::Dprev()
// Return the next cw edge around (into) the destination of the current edge.
{
        return invRot()->Onext()->invRot();
}

inline Edge* Edge::Lnext()
// Return the ccw edge around the left face following the current edge.
{
        return invRot()->Onext()->Rot();
}

inline Edge* Edge::Lprev()
// Return the ccw edge around the left face before the current edge.
{
        return Onext()->Sym();
}

inline Edge* Edge::Rnext()
// Return the edge around the right face ccw following the current edge.
{
        return Rot()->Onext()->invRot();
}

inline Edge* Edge::Rprev()
// Return the edge around the right face ccw before the current edge.
{
        return Sym()->Onext();
}

/************** Access to non-topological info ******************************/

inline Data* Edge::Org()
{
        return data;
}

inline Data* Edge::Dest()
{
        return Sym()->data;
}

inline const Point2d& Edge::Org2d() const
{
        return *data;
}

inline const Point2d& Edge::Dest2d() const
{
        return (num < 2) ? *((this + 2)->data) : *((this - 2)->data);
}

inline void Edge::EndPoints(Data* _or, Data* de)
{
        data = _or;
        Sym()->data = de;
}

inline Boolean Edge::isConstrained()
{
        return QEdge()->isConstrained();
}

inline void Edge::Constrain()
{
        QEdge()->Constrain();
}

#endif /* QUADEDGE_H */
