/***************************************************************************
* Main.C                                                                   *
*                                                                          *
*   Changes:                                                               *
*                                                                          *
*       08/06/99  arvo    Restructuring and cleanup.                       *
*       12/01/96  guo     Initial codiing.                                 *
*                                                                          *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo                                           *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <tcl.h>
#include <tk.h>
#include <rec/Train.h>
#include <util/Args.h>

static int debug = 0;
static int mode  = interactive_mode;

static void progress( int, const char *msg )
    {
    if( debug > 0 ) 
        {
        cout << msg << endl;
        }
    }

int main( int argc, char *argv[] )
    {
    const char *data_dir = ".";
    const char *font_dir = "";

    // Make the command line arguments available to any module
    // that wants to interrogate them.
    CommandLine args( argc, argv );

    // Handle the "version" flag.
    if( args.GetSwitch( "v" ) )
        {
        cout << "Version of Software       = " << SoftwareVersion << endl;
        cout << "Version of Training Files = " << TD_Version      << endl;
        cout << "Version of Feature Matrix = " << FM_Version      << endl;
        exit(0);
        }

    // Handle the "debug" flag.
    if( args.GetSwitch( "x" ) )
        {
        debug = 1;
        }

    // Handle the "data directory" flag.
    if( args.GetSwitch( "d" ) )
        {
        data_dir = args.GetArg( "d" );
        }

    // Handle the "font directory" flag.
    if( args.GetSwitch( "f" ) )
        {
        font_dir = args.GetArg( "f" );
        }

    // Handle the "regression" flag.
    if( args.GetSwitch( "r" ) )
        {
        mode = regression_only;
        }

    // Handle the "thumbnails" flag.
    if( args.GetSwitch( "t" ) )
        {
        mode = thumbnails_only;
        }

    // Handle the "help" flag.
    if( args.GetSwitch( "h" ) )
        {
        cout << endl;
        cout << "Usage: Train { -[dfhrtvx] [option] }*" << endl            << endl;
        cout << "  -d <dname> Read training data from this directory."     << endl;
        cout << "  -f <dname> Read font descriptions from this directory." << endl;
        cout << "  -h         Print these instructions and quit."          << endl;
        cout << "  -r         Run regression test and quit."               << endl;
        cout << "  -t         Show thumbnail images only."                 << endl;
        cout << "  -v         Print the version numbers and quit."         << endl;
        cout << "  -x         Enable debug output."                        << endl;
        cout << endl;
        exit(0);
        }

    // Extract the path qualifier that was used to invoke this program.
    static char base_path[ Max_Path_Length ];
    getPath( argv[0], base_path );

    // Create a "Train" object, which will create a Tcl/Tk interface.
    // Pass it the path to the directory where it should look for the
    // Tcl file(s) to load.
    Train trainer( base_path );

    // Set the progress meter, load the files, and start the interface.
    // Do not pass any command line arguments on to Tck/Tk, since they are
    // all handled by the CommandLine class.
    trainer.SetProgressMeter( progress );
    trainer.Run( 1, argv, data_dir, font_dir, mode );

    return 0;
    }

