/***************************************************************************
* List.C                                                                   *
*                                                                          *
* Base class for objects that may be inserted into a list.                 *
*                                                                          *
*   Changes:                                                               *
*                                                                          *
*       12/11/97    arvo    Added PromoteTail to make scheduling fair.     *
*       12/08/97    arvo    Broken out of Util.C                           *
*       07/01/97    arvo    Rewritten as a base class for Glyphs.          *
*       06/22/97    arvo    Initial coding.                                *
*                                                                          *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo                                           *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#include <string.h>
#include <util/List.h>

ListItem::ListItem() 
    { 
    next  = NULL; 
    back  = NULL; 
    owner = NULL; 
    }

ListItem::~ListItem() 
    { 
    next  = NULL; 
    back  = NULL; 
    owner = NULL; 
    }

List::List()
    {
    first = NULL;
    last  = NULL;
    size  = 0;
    }

List::~List() 
    { 
    Clear(); 
    }

List &List::operator<<( ListItem *node )
    {
    // Insert the given node at the tail of the doubly-linked list, 
    // updating the last pointer.
    //
    if( node  == NULL ) return *this;
    if( first == NULL ) first = node;
    if( last  != NULL ) last->next = node;
    node->owner = this;
    node->back  = last;
    node->next  = NULL;
    last = node;
    size++;
    return *this;
    }

ListItem *List::operator>>( ListItem *node )
    {
    // Remove the given node from the doubly-linked list, updating the 
    // pointer to the head or tail of the list as necessary.  Return NULL 
    // if the node was not in the List to begin with.
    //
    if( node == NULL || node->Owner() != this ) // Not a node in the List.
        {
        return NULL;
        }
    else if( node == first )
        {
        first = node->next;
        if( first == NULL ) last = NULL;
        else first->back = NULL;
        }
    else if( node == last )
        {
        last = node->back;
        last->next = NULL;
        }
    else // The general case.
        { 
        node->back->next = node->next;
        node->next->back = node->back;
        }
    node->next  = NULL;
    node->back  = NULL;
    node->owner = NULL;
    size--;
    return node;
    }

void List::Clear()
    {
    // Delete all the entries currently in the List.
    //
    ListItem *q = First(); 
    while( q != NULL )
        {
        ListItem *qnext = q->Next();
        delete q;
        q = qnext;
        }
    first = NULL;
    last  = NULL;
    size  = 0;
    }

