/***************************************************************************
* Timer.C                                                                  *
*                                                                          *
* Basic clock utilities.                                                   *
*                                                                          *
*   HISTORY                                                                *
*      Name    Date        Description                                     *
*                                                                          *
*      arvo    04/02/95   Initial implementation.
*                                                                          *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo                                           *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#include <unistd.h>  // Required on SGI.
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <sys/time.h>
#include <sys/times.h>
#include <util/Timer.h>

void Timer::Sleep( float sec )
    {
    usleep( (unsigned long)( sec * 1000000L ) );
    }

struct TimerData {  // This hides system-related definitions.
    timeval real_start;
    tms     cpu_start;
    clock_t clk_start;
    };

Timer::Timer() 
    { 
    data = new TimerData;
    Start(); 
    }

Timer::~Timer()
    {
    TimerData *td = (TimerData *)data;
    delete td;
    data = NULL;
    }

void Timer::Start()
    {
    TimerData *td = (TimerData *)data;
    struct timezone zone;
    gettimeofday( &td->real_start, &zone );
    times( &td->cpu_start );
    td->clk_start = clock();
    }

static long TimeDiff( const timeval &T1, const timeval &T2 )
    {
    long s = T2.tv_sec  - T1.tv_sec;
    long u = T2.tv_usec - T1.tv_usec;
    if( u < 0 ) 
        {
        u += 1000000;
        s -= 1;
        }
    return s * 1000000 + u;
    }

long Timer::RealMicroseconds() const
    {
    TimerData *td = (TimerData *)data;
    struct timeval  now;
    struct timezone zone;
    gettimeofday( &now, &zone );
    return TimeDiff( td->real_start, now );
    }

long Timer::CPU_Microseconds() const
    {
    TimerData *td = (TimerData *)data;
    struct tms now;
    times( &now );
    return now.tms_utime - td->cpu_start.tms_utime;
    }

long Timer::SYS_Microseconds() const
    {
    TimerData *td = (TimerData *)data;
    clock_t now = clock();
    return now - td->clk_start;
    }

float Timer::RealMilliseconds() const
    {
    return RealMicroseconds() / 1000.0;
    }

float Timer::CPU_Milliseconds() const
    {
    return CPU_Microseconds() / 1000.0;
    }

float Timer::SYS_Milliseconds() const
    {
    return SYS_Microseconds() / 1000.0;
    }

double Timer::RealSeconds() const
    {
    return RealMicroseconds() / 1000000.0;
    }

double Timer::CPU_Seconds() const
    {
    return CPU_Microseconds() / 1000000.0;
    }

double Timer::SYS_Seconds() const
    {
    return SYS_Microseconds() / 1000000.0;
    }

/*
TTime::TTime(const struct timeval& tv, const short UTCflag)
{
  hours    = (long)( (tv.tv_sec  / 3600L) % 24L );
  minutes  = (long)( (tv.tv_sec  /   60L) % 60L );
  seconds  = (long)(  tv.tv_sec  %   60L        );
  useconds = (long)(  tv.tv_usec                );
}

double Timer::CPU_Microseconds() const
    {
    struct tms now;
    times( &now );
    double diff = now.tms_utime - cpu_start.tms_utime;
    return diff / CLK_TCK;
    }

*/

