/***************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#ifndef __CharRec_h
#define __CharRec_h

// Smartboard includes
#include <util/Vec2.h>		// 2-D vectors
#include <util/Multiline.h>	// multilines
#include <rec/FeatureRecognizer.h>	// the stroke recogniser
#include <util/Box2.h>		// 2-D boxes.

#include "intlist.h"
#include "combos.h"
#include "common.h"
#include "max_strokes.h"
#include "rec_data.h"
#include "intersect.h"

struct PreCalcData_T {
  char *WhatChar;
  double Confidence;
  int NumberOfStrokes;
  Box2d BoundingBox;
  Multiline Stroke;
  char OtherChoices[512];  // Other choices suggested by the recogniser
};

typedef struct PreCalcData_T PreCalcData;

class CharRec
{
 private:
  FeatureRecognizer   *Recogniser;
  int                 *StrokeLength;
  IntList              StrokeFromCollections[MAX_STROKES_TO_PROCESS_AT_ONCE];
  int                  StrokeCount;
  int                  TotalStrokeCount;
  Multiline           *StrokeMultiline;
  char               **ComboStrings;
  Combos               StrokeCombos;
  ComboSequence       *CurrentSeq;
  double               ThisConf;
  int                  BestSeqNum;
  double               BestSeqConf;
  int                  StrokesCountedInRecognisedGlyphs;
  int                  StrokesUsed;
  int                  SeqPos;
  PreCalcData **PreCalc;
  Profile              LetterProf;
  int                  MaxStrokeCount;
  char                 StuffFiguredOut[4096];

 private:
  void Glue_Together_Strokes(int StartStrokeNum, 
			     int NumberOfStrokes,
			     const Multiline *StrokeM,
			     Multiline &ML);
  bool MultilinesOverlap(Multiline &M1, Multiline &M2);
  void Allocate_Do_Recognise_Memory();
  void Tidy_Up_Memory();
  void Find_Best_Combo();
  void Do_Precalculate();
  void Generate_Stroke_Combos();
  void Determine_Overlaps();
  void Display_All_Strokes();
  void Initialise_Strokes();
  void GoDoYourFunkyThing();

 public:
  CharRec();
  ~CharRec();
  void SaveStuffFiguredOut();

  void DoNormalRec(int NumberOfMultilines,
		   int *OriginalCollection,
		   int *PointCounts,
		   int *Points);
  void GetResultForTclTk(char *Dest);
};


#endif
