/* *************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
// Common nifty and useful stuff

#ifndef _COMMON_H_
#define _COMMON_H_

#define MAX_STROKES_TO_PROCESS_AT_ONCE  13
#define MAX_POINTS_PER_STROKE           256
#define MAX_STRING_LENGTH 16          /* maximum length of string expected to be returned by What_It_Is */

typedef double Coordinate_Value;

struct TWO_D_T {
  Coordinate_Value x;
  Coordinate_Value y;
};

struct THREE_D_T {
  Coordinate_Value x;
  Coordinate_Value y;
  Coordinate_Value z;
};

typedef struct TWO_D_T Position_2D;
typedef struct THREE_D_T Position_3D;



#ifndef MIN
#define MIN(x,y)       ( (x)>(y) ? (y):(x) )
#endif

#ifndef MAX
#define MAX(x,y)       ( (x)<(y) ? (y):(x) )
#endif

#ifndef ABS
#define ABS(x)         ( ((x)>0) ? (x):(-(x)) )
#endif

// concatentate two strings, putting a space between them
#define STRSPACECAT(s1,s2) {strcat(s1," ");strcat(s1,s2);}

#define RAD2DEG(x)     ( (x)*57.295779 )
#define DEG2RAD(x)     ( (x)*0.017453293 )
#define BETWEEN(a,b,c) ( ((b)<=(a))&&((a)<=(c)) ) // "a" is between "b" and "c" (inclusive)
#define SGN(x)         ( ((x)>0) ? (1):(-(1)) )

// ROTATE will rotate a point on a 2D plane
#define ROTATE(x,y,Rotate) { double nx,ny; nx = x*cos(Rotate) - y*sin(Rotate);  ny = x*sin(Rotate) + y*cos(Rotate); x = nx; y = ny; }

#define char_to_lower(x)  ( (((x)>='A')&&((x)<='Z') )?( (x) + 32):(x) )


#define AR_MIN 0.5 // minimum aspect ratio we consider for Average_Glyph_Size
#define AR_MAX 2.0 // maximum aspect ratio we consider for Average_Glyph_Size 


// Functions

long Factorial(int N);
int CombinatorialFunction(int N,int R);

double **Calloc_2D_double(int y, int x);
double *Calloc_double(int x);

void Free_2D_double(double **Place, int y);
void Free_double(double *Place);


char **Calloc_2D_char(int y, int x);
char *Calloc_char(int x);

void Free_2D_char(char **Place, int y);
void Free_char(char *Place);

int ***Calloc_3D_int(int z, int y, int x);
int **Calloc_2D_int(int y, int x);
int *Calloc_int(int x);

void Free_3D_int(int ***Place, int z, int y);
void Free_2D_int(int **Place, int y);
void Free_int(int *Place);

char *Font_File_Base_Directory(void);
void Wait_For_Key();

#endif
