/* *************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
*   HISTORY:                                                               *
*     Original Version, 1999                                               *
*     March 4, 2002, Richard Zanibbi                                       *
*       -commented out includes	of latex_preview.h and                     *
*        intersect.h; these are unused, and have been removed              *
*        from the distribution; Do_LaTeX_Preview function                  *
*        has also been removed.                                            *
*       -thumb.h and corresponding thumbnail code has                      *
*        also been commented out and removed.                              *
*     March 15, 2002, Richard Zanibbi                                      *
*       - changed my_init to use an environment variable to set the        *
*         base path for FFES again.  Also removed warning about            *
*         FONTFILEDIR in main (now using CIT_DIR: see common.cc)           *
***************************************************************************/

/******************************************************************************
 * RESEARCH PUBLICATIONS ABOUT FFES:
 
	1. Steve Smithies. Freehand Formula Entry System. Master's thesis, 
	   University of Otago, Dunedin, New Zealand, May 1999. 
	   
	2. S. Smithies, K. Novins, J. Arvo. "A handwriting-based equation editor," 
	   In Proc. Graphics Interface, Kingston, Ontario, Canada, June 1999. 
	   
	3. S. Smithies, K. Novins, and J. Arvo. Equation entry and editing via 
	   handwriting and gesture recognition. Behaviour and 
	   Information Technology, 20(1):53-67, 2001. 

	4. Richard Zanibbi, Kevin Novins, James Arvo and Katherine Zanibbi. 
	   ``Aiding Manipulation of Handwritten Mathematical Expressions through 
	   Style-Preserving Morphs'', In Proceedings of Graphics Interface 2001, 
	   Ottawa, Canada, June 2001, pp. 127-134. 

******************************************************************************/


// Steve's character recogniser!

#include <strings.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <pthread.h>

#include <iostream.h>

// Tcl/Tk includes
#include <tcl.h>
#include <tk.h>

// Smartboard includes
#include <util/Vec2.h>		// 2-D vectors
#include <util/Multiline.h>	// multilines
#include <rec/Recognizer.h>	// the stroke recogniser
#include <util/Box2.h>		// 2-D boxes.

// My includes
#include "charrec.h"
#include "MapWindow.h"

// Global variables.
static CharRec *CR = 0;
char MainTkInterfaceFile[256] = "";

int start_recogniser(ClientData client_data,
		     Tcl_Interp* interp,
		     int argc,
		     char *argv[])
{
  if (CR)
    delete CR;

  CR = new CharRec();

  return TCL_OK;
}

int close_recogniser(ClientData client_data,
		     Tcl_Interp* interp,
		     int argc,
		     char *argv[])
{
  delete CR;
  CR = 0;

  return TCL_OK;
}

int recognise_this(ClientData client_data,
		     Tcl_Interp* interp,
		     int argc,
		     char *argv[])
{
  if (argc != 5)
    {
      sprintf(interp->result,"Wrong number of arguments in recognise_this in ffes.cc");
      return TCL_ERROR;
  }

  int NumberOfMultilines = atoi(argv[1]);
  int *OriginalCollection = new int[NumberOfMultilines];
    int *PointCounts = new int[NumberOfMultilines];

  char *p;
  int c = 0;
  p = strtok(argv[2], " ");
  while (p)
    {
      OriginalCollection[c++] = atoi(p);
      p = strtok(0, " ");
    }
  assert(c == NumberOfMultilines);


  int TotalPoints = 0;
  c = 0;
  p = strtok(argv[3], " ");
  while (p)
    {
      PointCounts[c] = atoi(p);
      TotalPoints += PointCounts[c];
      c++;
      p = strtok(0, " ");
    }

  assert(c == NumberOfMultilines);

  int *Points = new int[TotalPoints * 2];

  c = 0;
  p = strtok(argv[4], " ");
  while (p)
    {
      Points[c++] = atoi(p);
      p = strtok(0, " ");
    }
  assert( c == (TotalPoints * 2) );

  CR->DoNormalRec(NumberOfMultilines,
		  OriginalCollection,
		  PointCounts,
		  Points);

  delete PointCounts;
  delete Points;

  char res[2048];
  CR->GetResultForTclTk(res);
  sprintf(interp->result,res);

  return TCL_OK;
}

int boxes_overlap(ClientData client_data,
		  Tcl_Interp* interp,
		  int argc,
		  char *argv[])
{
  char res[1024];
  if (argc!=9) {
    sprintf(interp->result,"Wrong number of arguments in boxes_overlap in ffes.cc");
    return TCL_ERROR;
  }

  Vec2 tl1(atoi(argv[1]),atoi(argv[2]));
  Vec2 br1(atoi(argv[3]),atoi(argv[4]));
  Vec2 tl2(atoi(argv[5]),atoi(argv[6]));
  Vec2 br2(atoi(argv[7]),atoi(argv[8]));

  Box2d Box1(tl1,br1);
  Box2d Box2(tl2,br2); 
  
  sprintf(res,"%d",Box1.dist(Box2)==0?1:0);
  Tcl_SetResult(interp, res, TCL_VOLATILE);

  return TCL_OK;
}

int LineIntersectTest(ClientData client_data,
		      Tcl_Interp* interp,
		      int argc,
		      char *argv[])
{
  if (argc!=9) {
    sprintf( interp->result, "wrong number of arguments for ffes.cc:LineIntersectTest (want AxAy,BxBy,CxCy,DxDy)");
    return TCL_ERROR;
  }

  Position_2D A;
  Position_2D B;
  Position_2D C;
  Position_2D D;

  A.x = atoi(argv[1]);
  A.y = atoi(argv[2]);
  B.x = atoi(argv[3]);
  B.y = atoi(argv[4]);
  C.x = atoi(argv[5]);
  C.y = atoi(argv[6]);
  D.x = atoi(argv[7]);
  D.y = atoi(argv[8]);
  
  int res = LineIntersectTest(A,B,C,D);
  char tclres[1024];
  
  sprintf(tclres,"%d",res?1:0);
  Tcl_SetResult(interp, tclres, TCL_VOLATILE);
  return TCL_OK;
}


int my_init( Tcl_Interp *interp )
{
  if (Tcl_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }
  if (Tk_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }

  Tcl_CreateCommand( interp, "start_recogniser", start_recogniser, NULL, NULL);
  Tcl_CreateCommand( interp, "close_recogniser", close_recogniser, NULL, NULL);
  Tcl_CreateCommand( interp, "recognise_this", recognise_this, NULL, NULL);

  Tcl_CreateCommand( interp, "Boxes_Overlap", boxes_overlap, NULL, NULL);
  Tcl_CreateCommand( interp, "LineIntersectTest",           LineIntersectTest,           NULL, NULL);

  if (MapWindow_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }
 
  // R. Zanibbi, March 15, 2002: use an environment variable so
  // that we can run FFES places other than the ffes_0.2 directory.
  char * FFESDir = getenv("FFES_DIR");
  char * InterfaceDir = "/Tk_Interface/Interface.tk";
  
  if (FFESDir == NULL) {
	  cerr << "ERROR: FFES_DIR needs to be defined in the environment.\nIt"
		  << " should point to the base directory of FFES (e.g. <path>/ffes_0.2)\n";
	  exit(1);
  }

  strcat(MainTkInterfaceFile,FFESDir);
  strcat(MainTkInterfaceFile,InterfaceDir);

  Tcl_SetVar( interp, "tcl_rcFileName", MainTkInterfaceFile, TCL_GLOBAL_ONLY );

  return 0;

}


int main(int argc, char **argv)
{
  
  Tk_Main( argc, argv, my_init );
  return 0;
}
