/* *************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#include "common.h"
#include "intersect.h"

// //////////////////////////////////////////////////////////////////////////////
// Routine for testing to see if two line segments (a~ to b~) and (c~ to d~) cross
//
// Credit to Hugh Malan for figuring out how to do this.
//

#define DIFF(A,B,C) C.x = B.x - A.x;C.y = B.y - A.y
#define CROSS(A,B,C) C.x = -(A.z *B.y) + A.y* B.z; C.y = A.z* B.x - A.x* B.z; C.z = -(A.y* B.x) + A.x* B.y
#define SET2D3D(A,B) B.x = A.x;B.y=A.y;B.z=0
#define ZSIGN(A) ((A.z)>0)?1:0

int PointOnLine( Position_2D P, Position_2D Q, Position_2D T )
{
   // Checks to see if the point T is on the line PQ.  
   
   // Algorithm based on the triangle inequality.  If T is on PQ then
   // length(PQ) = length(PT) + length(TQ).  Otherwise the sum is greater.
   // This routine returns an intersection if the two sides of the equation
   // differ more than a specified tolerance.  Uses length squared rather than
   // length.

   return(
        (
	((P.x - T.x) * (P.x - T.x) + (P.y - T.y) * (P.y - T.y))	+
	((Q.x - T.x) * (Q.x - T.x) + (Q.y - T.y) * (Q.y - T.y))	-
	((P.x - Q.x) * (P.x - Q.x) + (P.y - Q.y) * (P.y - Q.y))	
        )
	< 1.0
   );
}

int LineIntersectTest(Position_2D A,
		      Position_2D B,
		      Position_2D C,
		      Position_2D D)
{
  // Check to see if endpoints overlap first.  It's cheap. (KLN)
  if ( (A.x == C.x) && (A.y == C.y ) ) return 1; 
  if ( (A.x == D.x) && (A.y == D.y ) ) return 1; 
  if ( (B.x == C.x) && (B.y == C.y ) ) return 1; 
  if ( (B.x == D.x) && (B.y == B.y ) ) return 1; 

  // Check for degenerate cases where one "line" is actually a point (KLN)
  if ( (A.x == B.x) && (A.y == C.y) ) return (PointOnLine(C,D,A));
  if ( (C.x == D.x) && (C.y == D.y) ) return (PointOnLine(A,B,C));

  // If we make it here there are two bona-fide line segments.
  // Use the Smithies-Malan test. (KLN)
	
  Position_2D AB;
  Position_2D CD;
  Position_2D AC;
  Position_2D AD;
  Position_2D CA;
  Position_2D CB;
  Position_3D AB3D;
  Position_3D CD3D;
  Position_3D AC3D;
  Position_3D AD3D;
  Position_3D CA3D;
  Position_3D CB3D;
  Position_3D R;
  int sign1,sign2,sign3,sign4;

  DIFF(A,B,AB);
  DIFF(C,D,CD);
  DIFF(A,C,AC);
  DIFF(A,D,AD);
  DIFF(C,A,CA);
  DIFF(C,B,CB);

  SET2D3D(AB,AB3D);  
  SET2D3D(CD,CD3D);  
  SET2D3D(AC,AC3D);  
  SET2D3D(AD,AD3D);  
  SET2D3D(CA,CA3D);  
  SET2D3D(CB,CB3D);  

  CROSS(AB3D,AC3D,R);
  sign1 = ZSIGN(R);

  CROSS(AB3D,AD3D,R);
  sign2 = ZSIGN(R);

  CROSS(CD3D,CA3D,R);
  sign3 = ZSIGN(R);

  CROSS(CD3D,CB3D,R);
  sign4 = ZSIGN(R);

  if ( (sign1 != sign2) && (sign3 != sign4)) {
    return 1;
  } else {
    return 0;
  };
}
