/* *************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>
//#include <malloc.h>
#include <string.h>

#include "max_strokes.h"

// is_dir from smartboard code
int is_dir( short mode )
{
  return ( ( mode & S_IFDIR ) == S_IFDIR );
}

int Is_A_Font_File(char *Fn)
{
  FILE *f;
  char temp[255];
  int ret;

  f = fopen(Fn,"rt");
  
  if (f==NULL) {
    sprintf(temp,"fopen(\"%s\",\"rt\")",Fn);
    perror(temp);
    return 0;
  }

  fgets(temp,255,f);

  // Decide if it is a font file or not by looking at the first line of
  // each file.
  ret = (strcmp(temp,"[Training Data: Version 3]\n"))?0:1;

  fclose(f);

  return ret;
}

int Max_Strokes_In_This_Font_File(char *Fn)
{
  FILE *f;
  char temp[255];
  int ret = -1;

  f = fopen(Fn,"rt");
  
  if (f==NULL) {
    sprintf(temp,"fopen(\"%s\",\"rt\")",Fn);
    perror(temp);
    return 0;
  }

  while (!feof(f)) {
    char *p;
    int v;
    char ComponentsString[] = "components = ";
    fgets(temp,255,f);

    p = strstr(temp, ComponentsString);
    if (p!=NULL) {
      v = atoi(p+strlen(ComponentsString));
      if (v>ret) {
	ret = v;
      }
    }
  }
  fclose(f);
  if (ret == -1) {
    fprintf(stderr,"Unable to determine maximum strokes.  Assuming 4.\n");
    ret = 4;
  }
  return ret;
}

void Do_Trawl_For_Fonts(char *path, int Recursive, char ***FileName, int *FileNameCount)
{
  DIR *d;
  dirent *bob;

  d = opendir(path);
  if (d==NULL) {
    perror("opendir");
    exit(1);
  } 
  while ( (bob = readdir(d)) != NULL) {
    struct stat filestat;
    char fullpath[255];

    sprintf(fullpath,"%s/%s",path,bob->d_name);

    lstat(fullpath, &filestat);
    if ( strcmp(bob->d_name,".")&&
	 strcmp(bob->d_name,"..")&&
	 is_dir(filestat.st_mode) ) {
      Do_Trawl_For_Fonts(fullpath, 1, FileName, FileNameCount);
    } else {
      if ( strcmp(bob->d_name,".")&&
	   strcmp(bob->d_name,"..")&&
	   Is_A_Font_File(fullpath) ) {
	(*FileNameCount)++;
	*FileName = (char **)realloc(*FileName,*FileNameCount*sizeof(char *));
	(*FileName)[*FileNameCount-1] = strdup(fullpath);
      }
    }
  }
  closedir(d);
}

int Trawl_For_Fonts(char *path, int Recursive, char ***FileNamePlace)
{
  int FileNameCount;

  FileNameCount = 0;
  *FileNamePlace = (char **)NULL;
  Do_Trawl_For_Fonts(path, Recursive, FileNamePlace, &FileNameCount);

  return FileNameCount;
}

// Free memory allocated by Trawl...
void FreeAll(char **P, int n)
{
  for (int i=0;i<n;i++) {
    free(P[i]);
  }

  free(P);
}

int Max_Strokes_In_Font(char *FontDir)
{
  char **FileName;

  int fnc;
  int max = -1;

  fnc = Trawl_For_Fonts(FontDir, 1, &FileName);

  for (int i=0;i<fnc;i++) {
    int mymax;
    mymax = Max_Strokes_In_This_Font_File(FileName[i]);
    //    printf("%d %s\n",mymax,FileName[i]);
    if (mymax > max) {
      max = mymax;
    }
  }

  FreeAll(FileName, fnc);

  if (max == -1) {
    fprintf(stderr,"Unable to determine maximum strokes.  Something has gone horribly wrong.  Assuming 4.\n");
    max = 4;
  }

  return max;
}
